package com.bwie.auth.service.impl;

import com.bwie.auth.feign.AuthFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.auth.util.MsgUtil;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginReq;
import com.bwie.common.domain.response.JwtRes;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.result.Result;
import com.bwie.common.utils.IdUtils;
import com.bwie.common.utils.JwtUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    private final MsgUtil msgUtil;
    private final AuthFeign authFeign;
    private final RedisCache redisCache;
    private final HttpServletRequest request;

    public AuthServiceImpl(MsgUtil msgUtil, AuthFeign authFeign, RedisCache redisCache, HttpServletRequest request) {
        this.msgUtil = msgUtil;
        this.authFeign = authFeign;
        this.redisCache = redisCache;
        this.request = request;
    }

    @Override
    public String sendcode(String userTel) {
        msgUtil.sendMsg(userTel,"1111");
        redisCache.setCacheObject("tel_"+userTel,"1111");
        return "1111";
    }

    @Override
    public JwtRes login(LoginReq loginReq) {
        Result<User> findtel = authFeign.findtel(loginReq.getUserTel());
        Assert.isTrue(findtel.isSuccess(),"登陆失败");
        User data = findtel.getData();
        Assert.notNull(data,"电话不存在");
        Assert.isTrue(redisCache.hasKey("tel_" + loginReq.getUserTel()),"验证码过期");
        String cacheObject = redisCache.getCacheObject("tel_" + loginReq.getUserTel());
        Assert.isTrue(loginReq.getUserCode().equals(cacheObject),"验证码错误");
        HashMap<String, Object> map = new HashMap<>();
        String s = IdUtils.genId();
        map.put(JwtConstants.USER_KEY,s);
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+s,data,TokenConstants.EXPIRATION, TimeUnit.SECONDS);

        return JwtRes.builder()
                .expireTime(TokenConstants.EXPIRATION)
                .token(token)
                .build();
    }

    @Override
    public User info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);

        return cacheObject;
    }
}
