package com.bwie.gateway.filters;

import com.bwie.common.constant.TokenConstants;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.concurrent.TimeUnit;

@Component
public class AuthFilter implements GlobalFilter, Ordered {
    private final RedisCache redisCache;
    private final IgnoreWhiteConfig ignoreWhiteConfig;

    public AuthFilter(RedisCache redisCache, IgnoreWhiteConfig ignoreWhiteConfig) {
        this.redisCache = redisCache;
        this.ignoreWhiteConfig = ignoreWhiteConfig;
    }
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        if(StringUtils.matches(path,ignoreWhiteConfig.getWhites())){
            return chain.filter(exchange);
        }

        String first = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if(null==first){
            return GatewayUtils.errorResponse(exchange,"token不为空");
        }

        Claims claims = JwtUtils.parseToken(first);
        if(null==claims){
            return GatewayUtils.errorResponse(exchange,"token格式错误");
        }
        String userKey = JwtUtils.getUserKey(claims);
        if(!redisCache.hasKey(TokenConstants.LOGIN_TOKEN_KEY+userKey)){
            return GatewayUtils.errorResponse(exchange,"token过期");
        }

        redisCache.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,15, TimeUnit.MINUTES);



        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
