package com.bwie.car.service;

import com.alibaba.fastjson.JSON;
import com.bwie.car.mapper.CarMapper;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.CarRes;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static net.sf.jsqlparser.parser.feature.Feature.show;

@Service
public class CarServiceImpl implements CarService{
    private final CarMapper carMapper;
    private final HttpServletRequest request;
    private final StringRedisTemplate redisTemplate;
    private final RedisCache redisCache;

    public CarServiceImpl(CarMapper carMapper, HttpServletRequest request, StringRedisTemplate redisTemplate, RedisCache redisCache) {
        this.carMapper = carMapper;
        this.request = request;
        this.redisTemplate = redisTemplate;
        this.redisCache = redisCache;
    }

    @Override
    public Result addcar(List<String> carReq) {
        for (String c : carReq) {
            String[] split = c.split("-");
            Long id= Long.valueOf(split[0]);
            Integer num= Integer.valueOf(split[1]);
            carMapper.addcar(id,num);
        }



        return Result.success();
    }

    @Override
    public Result show() {
        if(!redisCache.hasKey("car_"+info().getUserId())){
            List<CarRes> show=carMapper.show(info().getUserId());
            HashSet<CarRes> objects = new HashSet<>();

            for (CarRes carRes : show) {
                objects.add(carRes);
            }
            redisCache.setCacheSet("car_"+info().getUserId(),objects);
        }
        Set<CarRes> cacheSet = redisCache.getCacheSet("car_" + info().getUserId());

        return Result.success(cacheSet);
    }
    public User info() {
        String header = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(header);
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return cacheObject;
    }
}
