package com.bwie.goods.service;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.domain.Goods;
import com.bwie.common.domain.request.GoodsReq;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.goods.mapper.GoodsMapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements GoodsService{

    private final GoodsMapper goodsMapper;
    private final RestHighLevelClient client;

    public GoodsServiceImpl(GoodsMapper goodsMapper, RestHighLevelClient client) {
        this.goodsMapper = goodsMapper;
        this.client = client;
    }

    @Override
    public void addb() {
        List<Goods> goods = goodsMapper.selectList(new LambdaQueryWrapper<Goods>());
        BulkRequest bulkRequest = new BulkRequest();
        for (Goods good : goods) {
            bulkRequest.add(new IndexRequest("goods").id(good.getGoodsId().toString())
                    .source(JSON.toJSONString(good), XContentType.JSON));
        }
        try {
            BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
            Assert.isTrue(!bulk.hasFailures(),"添加失败");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Result show(GoodsReq goodsReq) {
        SearchRequest searchRequest = new SearchRequest("goods");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = new BoolQueryBuilder();

        if(goodsReq.getGoodsBrand()!=null && !goodsReq.getGoodsBrand().equals("")){
            boolQueryBuilder.must(QueryBuilders.matchQuery("goodsBrand",goodsReq.getGoodsBrand()));
        }

        if(goodsReq.getGoodsName()!=null && !goodsReq.getGoodsName().equals("")){
            boolQueryBuilder.must(QueryBuilders.matchQuery("goodsName",goodsReq.getGoodsName()));
        }
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.sort(new FieldSortBuilder("goodsTime").order(SortOrder.DESC));
        searchSourceBuilder.from(0);
        searchSourceBuilder.size(goodsReq.getPageSize());
        Long total=0L;

        HighlightBuilder highlightBuilder = new HighlightBuilder();
        highlightBuilder.field("goodsName");
        highlightBuilder.preTags("<span style=\"color:red\">");
        highlightBuilder.postTags("</span>");
        searchSourceBuilder.highlighter(highlightBuilder);

        searchRequest.source(searchSourceBuilder);
        ArrayList<Goods> goods = new ArrayList<>();

        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
            for (SearchHit hit : search.getHits()) {
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                HighlightField name = highlightFields.get("goodsName");
                StringBuffer stringBuffer = new StringBuffer();
                if(name!=null){
                    for (Text fragment : name.getFragments()) {
                        stringBuffer.append(fragment);
                    }
                }
                String sourceAsString = hit.getSourceAsString();
                ObjectMapper objectMapper = new ObjectMapper();
                Goods goods1 = objectMapper.readValue(sourceAsString, Goods.class);
                if(name!=null){
                    goods1.setGoodsName(stringBuffer.toString());
                }
                goods.add(goods1);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return PageResult.toResult(total,goods);
    }


}























